#!/usr/bin/env python

"""
    STP Video Encoder iTunes U
    ==========================
    Video encoder for iTunes U that output H.264/AAC in MP4.
    
    License
    -------
    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org> & Josh Chesarek <josh@simplethoughtproductions.com>
    For full license information see libstpenc/__init__.py.
    
"""

import logging
import shutil

from .. import settings

from ..contrib import qtfaststart
from ..utils import execute, exec_with_progress

from ..encoders.base import Encoder

log = logging.getLogger("libstpenc.encoders.itunesu")

# Max allowed iTunesU size is 500 MB
MAX_ITUNESU_SIZE = 490 * 1024 ** 2
MAX_ITUNESU_BITRATE = 340

BIN_FFMPEG = settings.BIN_FFMPEG
BIN_X264 = settings.BIN_X264

class ItunesUEnc(Encoder):
    """
        H.264/AAC in MP4 Encoder for iTunes U/Ipod.
    """
    ext = "-itunesu.mp4"
    def encode(self):
        if self.metadata.duration * (self.vbitrate + self.abitrate) * 1024 / 8 > MAX_ITUNESU_SIZE: 
            bitrate = ((MAX_ITUNESU_SIZE / self.metadata.duration * 8 / 1024) - self.abitrate)
        
        else:
            bitrate = MAX_ITUNESU_BITRATE
        
        log.info("Creating H.264/AAC (MP4) conversion for Itunes U / IPod... Bitrate is: %d" %(bitrate) )
        
        common = "-y -i \"%s\" -deinterlace -s %dx%d -f rawvideo " \
                 "-pix_fmt yuv420p -r 25 - | %s --bitrate %i --no-cabac " \
                 "--merange 24 --threads auto " \
                 "--fps 25 -o \"video.264\" - %dx%d" % \
                 (self.infile, self.width, self.height, BIN_X264,
                  bitrate, self.width, self.height)
        pass1 = "%s --pass 1 --ref 1 --subme 1 --me dia" % common
        pass2 = "%s --pass 2 --ref 6 --subme 8 --me umh" % common
        
        if self.metadata.has_audio:
            log.info("Encoding AAC audio...")
            exec_with_progress("%s -y -i \"%s\" -vn -acodec libfaac -ab " \
                               "%ik \"audio.mp4\"" % (BIN_FFMPEG, self.infile,
                                                      self.abitrate),
                                                      self.metadata)
        
        log.info("Starting H.264 video pass 1 of 2...")
        exec_with_progress("%s %s" % (BIN_FFMPEG, pass1), self.metadata)
        
        log.info("Starting H.264 video pass 2 of 2...")
        exec_with_progress("%s %s" % (BIN_FFMPEG, pass2), self.metadata)
        
        if self.metadata.has_audio:
            log.info("Muxing audio/video to MP4...")
            execute("%s -y -i \"video.264\" -vcodec copy -i \"audio.mp4\" " \
                    "-acodec copy -f ipod \"%s\"" % (BIN_FFMPEG, self.outfile))
        else:
            shutil.move("video.mp4", self.outfile)
        
        log.info("Setting up file for streaming...")
        qtfaststart.process(self.outfile, self.outfile + "fast.mp4")
        shutil.move(self.outfile + "fast.mp4", self.outfile)
        
        log.debug("H.264 done.")

settings.FORMATS["itunesu"] = ItunesUEnc

