#!/usr/bin/env python

"""
    STP Video Encoder NTSC DVD
    ==========================
    NTSC DVD encoder. Outputs MPEG2 video and audio in separate files.
    
    License
    -------
    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org>
    For full license information see libstpenc/__init__.py.
    
"""

import logging

from .. import settings

from ..utils import exec_with_progress

from ..encoders.base import Encoder

log = logging.getLogger("libstpenc.encoders.ntscdvd")

# Max allowed DVD size is 3.9 GiB
MAX_DVD_SIZE = 3.9 * 1024 ** 3
MAX_DVD_BITRATE = 6 * 1024 ** 2

BIN_FFMPEG = settings.BIN_FFMPEG

class DVDEnc(Encoder):
    """
        Encode NTSC DVD compatible MPEG2 audio and video. This ignores the
        scale setting and always uses 720x480.
    """ 
    def encode(self):
        log.info("Creating MPEG2 conversion...")
        
        self.outaudio = self.prefix + "-dvd.mp2"
        self.outvideo = self.prefix + "-dvd.m2v"
        self.outfile = [self.outaudio, self.outvideo]
        
        bitrate = MAX_DVD_SIZE / self.metadata.duration * 8
        if bitrate + (384 * 1024) > MAX_DVD_BITRATE:
            bitrate = MAX_DVD_BITRATE
        else:
            bitrate -= (384 * 1024)
        bitrate = int(bitrate / 1024)
        
        common = "-y -i \"%s\" -target ntsc-dvd -b %dk -deinterlace -s " \
                 "720x480 -threads 4 -aspect 16:9" % (self.infile, bitrate)
        pass1 = "-an -pass 1"
        pass2 = "-an -pass 2"
        
        log.info("Converting audio to MP2...")
        exec_with_progress("%s -y -i \"%s\" -vn -acodec mp2 -ar 48000 -ab 384k " \
                           "\"%s\"" % (BIN_FFMPEG, self.infile, self.outaudio),
                                       self.metadata)
        
        log.info("Starting MPEG2 pass 1 of 2...")
        exec_with_progress("%s %s %s \"%s\"" % (BIN_FFMPEG, common, pass1,
                           self.outvideo), self.metadata)
        
        log.info("Starting MPEG2 pass 2 of 2...")
        exec_with_progress("%s %s %s \"%s\"" % (BIN_FFMPEG, common, pass2,
                           self.outvideo), self.metadata)
        
        log.debug("MPEG2 done.")

settings.FORMATS["ntsc-dvd"] = DVDEnc

