#!/usr/bin/env python

"""
    STP Video Encoder Metadata Extractor
    ====================================
    Extract information about input files, such as the duration.
    
    License
    -------
    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org>
    For full license information see libstpenc/__init__.py.
    
"""

import logging
import re
import subprocess

import settings

log = logging.getLogger("libstpenc.metadata")

# Regular expressions for matching metadata, formats, etc.
duration_re = re.compile("Duration: ([^,]*)")
audio_re = re.compile("Stream #[0-9]\.[0-9].*?Audio:")
dimensions_re = re.compile("\s+([0-9]*?)x([0-9]*?)[\s,]+(\[.*?\])?")

class Metadata(object):
    """
        Information about a media file, fetched via ffmpeg.
    """
    def __init__(self, filename):
        """
            Create a new metadata object and extract information about the
            given file.
        """
        self.filename = filename
        self.extract()
    
    def _reset(self):
        self.duration = 0
        self.has_audio = False
        self.dimensions = (0, 0)
        
    def extract(self):
        """
            Extract metadata info by calling ffmpeg. Attempts to find duration,
            whether the file contains audio, and the original video dimensions
            taking into account pixel aspect ratio.
        """
        self._reset()
        ffmpeg = settings.BIN_FFMPEG
        proc = subprocess.Popen("%s -i \"%s\"" % (ffmpeg, self.filename),
                                shell=True, stdout=subprocess.PIPE, 
                                stderr=subprocess.STDOUT)
        
        data = proc.stdout.read()
        
        result = duration_re.search(data, re.M)
        if result:
            parts = result.groups()[0].split(":")
            self.duration = int(parts[0]) * 60 * 60 + int(parts[1]) * 60 + \
                            int(float(parts[2]))
            log.debug("Duration is %d seconds" % self.duration)
        
        result = audio_re.search(data, re.M)
        if result:
            self.has_audio = True
            log.debug("Video has audio")
        
        result = dimensions_re.search(data, re.M)
        if result:
            w, h, ar = result.groups()[:3]
            w, h = int(w), int(h)
            
            if ar:
                ar = ar.strip("[").strip("]")
                parts = ar.split()
                for x in range(len(parts)):
                    if parts[x] == "PAR" and x < (len(parts) - 1):
                        try:
                            parx, pary = [float(val) for val in \
                                          parts[x + 1].split(":")]
                            w = w * (parx / pary)
                        except Exception, e:
                            print e
                        break
            
            self.dimensions = (w, h)
            log.debug("Video dimensions are %d x %d" % (w, h))
    
    def get_scaled_dimensions(self, scale):
        """
            Return scaled dimensions where scale is a string "widthxheight"
            where width and height can be a positive integer or -1 (-1 meaning
            auto). For example, to scale to a width of 480 but keep the original
            aspect ratio you would:
            
             >>> metadata.get_scaled_dimensions("480x-1")
            
            Note: only one dimension may be -1!
        """
        if type(scale) is tuple:
            sw, sh = [int(item) for item in scale]
        else:
            sw, sh = [int(item) for item in scale.split("x")]
        if sw == -1 and sh == -1:
            print "Both dimensions cannot be -1!"
            raise SystemExit(1)
        elif sw == -1:
            w, h = (int(self.dimensions[0] * (sh / float(self.dimensions[1]))), sh)
        elif sh == -1:
            w, h = (sw, int(self.dimensions[1] * (sw / float(self.dimensions[0]))))
        else:
            w, h = (sw, sh)
        
        if w % 2:
            w += 1
        if h % 2:
            h += 1
        
        return (w, h)

