#!/usr/bin/env python

"""
    STP Video Encoder Uploaders
    ===========================
    Tools to upload files using various protocols to various services.
    
    License
    -------
    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org>
    For full license information see libstpenc/__init__.py.
    
"""

import logging
import re

from ftp import transfer_ftp
from sftp import transfer_sftp

log = logging.getLogger("libstpenc.uploaders")

# Regular expression for matching upload information
upload_re = re.compile("^(?P<protocol>.*?)://((?P<user>.*?)(:(?P<password>.*?))?@)?(?P<hostname>.*?)(:(?P<path>.*))?$")

def upload(upload_info, filename):
    """
        Upload a file. upload_info must be a tuple of 
        ("proto://user:pass@host:basepath", "path") and filename the path to 
        the input file. The base name of the input file will be appended to 
        host:basepath/path when writing the file to the server.
        
        For example, to upload a file /home/dan/test.mp4 to 
        programmer-art.org:videos/test.mp4 via sftp using the current user 
        and SSH keys for authentication:
        
         >>> upload(("sftp://programmer-art.org", "videos"), "/home/dan/test.mp4")
        
    """
    dbase = os.path.basename(filename)
    
    protocol_info, remote_path = upload_info
    
    # Parse protocol information
    result = upload_re.search(protocol_info)
    if not result:
        log.error("Could not parse upload protocol info!")
        return
    
    protocol = result.group("protocol")
    user = result.group("user")
    password = result.group("password")
    hostname = result.group("hostname")
    base_path = result.group("path")
    
    if not base_path:
        base_path = ""
    
    dst = os.path.join(base_path, remote_path, dbase)
    
    # Automatically figure out username for secure protocols
    if protocol in ["sftp", "ftps"]:
        if not user:
            if os.environ.has_key("USER"):
                user = os.environ["USER"]
            else:
                log.error("No user set to upload as!")
                return
    
    # Transfer the file with the proper protocol!
    if protocol == "sftp":  
        transfer_sftp(hostname, user, password, filename, dst)
        
    elif protocol == "ftp":
        transfer_ftp(hostname, user, password, filename, dst, tls=False)
    
    elif protocol == "ftps":
        transfer_ftp(hostname, user, password, filename, dst)
    
    else:
        log.error("Unknown transfer protocol!")

