#!/usr/bin/env python

"""
    STP Video Encoder SFTP Uploader
    ===============================
    An uploader using SFTP (file transfer via ssh), capable of using stored
    DSA and RSA keys or passwords.
    
    License
    -------
    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org>
    For full license information see libstpenc/__init__.py.
    
"""

import logging
import os
import sys

import libstpenc.settings

from  libstpenc.utils import draw_progress, clear_progress

log = logging.getLogger("libstpenc.uploaders.sftp")

# Attempt to add sftp protocol
try:
    import paramiko
    libstpenc.settings.UPLOAD_PROTOCOLS.append("sftp")
except ImportError:
    sys.path.append(os.path.join("..", "contrib", "paramiko.zip"))
    try:
        import paramiko
        libstpenc.settings.UPLOAD_PROTOCOLS.append("sftp")
    except ImportError:
        log.warning("Couldn't import Secure FTP functionality!")

def transfer_sftp(host, user, password, src, dst):
    """
        Send a file src on the local machine to a file dst on host using the
        given user/pass via sftp.
    """
    
    def status(sent, total):
        percent = float(sent) / total
        info_str = " %s%%" % (str(int(percent * 1000) / 10))
        draw_progress(percent, info_str)
    
    log.info("Uploading %s to %s:%s" % (src, host, dst))
    
    # Connect to the server
    try:
        transport = paramiko.Transport(host)
    except socket.error:
        log.error("Unable to connect to sftp server %s!" % str(host))
        return
    
    # Use saved private and/or host keys?
    keyfile = os.path.expanduser("~/.ssh")
    if os.path.exists(os.path.join(keyfile, "id_dsa")):
        key = paramiko.DSSKey.from_private_key_file(os.path.join(keyfile, "id_dsa"))
    elif os.path.exists(os.path.join(keyfile, "id_rsa")):
        key = paramiko.RSAKey.from_private_key_file(os.path.join(keyfile, "id_rsa"))
    else:
        key = None
    
    hostkey = None
    hostkeyfile = os.path.expanduser("~/.ssh/known_hosts")
    if os.path.exists(hostkeyfile):
        hostkeys = paramiko.HostKeys(hostkeyfile)
        if hostkeys.has_key(host):
            if hostkeys[host].has_key("ssh-dss"):
                hostkey = hostkeys[host]["ssh-dss"]
            elif hostkeys[host].has_key("ssh-rsa"):
                hostkey = hostkeys[host]["ssh-rsa"]
            else:
                log.warning("Unknown host key type '%s' for host '%s'!" % \
                            (str(hostkeys[host]), str(host)))
    
    # Connect and authenticate
    if key:
        try:
            transport.connect(username = user, pkey = key, hostkey = hostkey)
        except paramiko.BadHostKeyException:
            log.error("The server did not return the expected host key. " \
                      "Your connection could be compromised and will be " \
                      "terminated!")
            transport.close()
            return
        except paramiko.SSHException:
            try:
                transport.auth_password(username = user, password = password)
            except paramiko.SSHException:
                log.error("Unable to authenticate with sftp server!")
                transport.close()
                return
    else:
        try:
            transport.connect(username = user, password = password,
                              hostkey = hostkey)
        except paramiko.BadHostKeyException:
            log.error("The server did not return the expected host key. " \
                      "Your connection could be compromised and will be " \
                      "terminated!")
            transport.close()
            return
        except paramiko.SSHException:
            log.error("Unable to authenticate with sftp server!")
            transport.close()
            return
    
    # Create an sftp client instance and send the file
    sftp = paramiko.SFTPClient.from_transport(transport)
    sftp.put(src, dst, status)
    
    clear_progress()
    
    # Close all connections
    sftp.close()
    transport.close()

