#!/usr/bin/env python

"""
    STP Video Encoder Utilities
    ===========================
    Utility methods used by the STP Video Encoder.
    
    License
    -------
    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org>
    For full license information see libstpenc/__init__.py.
    
"""

import logging
import os
import re
import subprocess
import sys
import tempfile
import time

log = logging.getLogger("libstpenc.utils")

time_re = re.compile("time=([0-9\.]*)")

def format_time(sec):
    """
        Return a nicely formatted time like HH:MM:SS from a given number of
        seconds.
        
            >>> format_time(0)
            '00:00:00'
            >>> format_time(62)
            '00:01:02'
            >>> format_time(300)
            '10:00:00'
        
        @type sec: int
        @param sec: The number of seconds to convert to a nicely formatted time
        @rtype: str
        @return: Nicely formatted time
    """
    s = ""
    
    sec = int(sec)
    
    div = sec / (60 * 60)
    if div:
        if div < 10:
            s += "0"
        s += str(div)
    else:
        s += "00"
    sec = sec % (60 * 60)
    s += ":"
    
    div = sec / 60
    if div:
        if div < 10:
            s += "0"
        s += str(div)
    else:
        s += "00"
    sec = sec % 60
    s += ":"
    
    if sec < 10:
        s += "0"
    s += str(sec)
    
    return s

def draw_progress(percent, info_str=""):
    """
        Draw a progress bar and optional info string, overwriting the previous
        one if it exists. This method 
        
        @type percent: number
        @param percent: The completed percentage of the progress bar
        @type info_str: str
        @param info_str: An extra string to add after the progress bar; use
                         this to append e.g. time left until completion
    """
    # Set the cursor back
    sys.stdout.write("\010" * 79)
    
    # Calculate the progress bar string, including the info_str passed in
    progbar_len = 78 - len(info_str) - 2
    bars = "=" * int((progbar_len * percent) + 0.5)
    if len(bars) > 2:
        bars = "8" + bars[1:]
    if len(bars):
        bars = bars[:-1] + "D"
    spaces = " " * int((progbar_len * (1.0 - percent)) + 0.5)
    progbar = "[" + bars + spaces + "]"
    
    # Write the progress bar + info_str
    output = progbar + info_str
    sys.stdout.write(output)
    sys.stdout.flush()

def clear_progress():
    """
        Clear the progress line. Call this after the progress has hit 100%.
    """
    # Clear line by backtracking, writing spaces, then backtracking again
    sys.stdout.write("\010" * 79)
    sys.stdout.write(" " * 79)
    sys.stdout.write("\010" * 79)
    sys.stdout.flush()

def execute(cmd):
    """
        Execute a command cmd.
    """
    log.debug("Running %s" % cmd)
    
    tmpfile, tmpfilename = tempfile.mkstemp()
    
    proc = subprocess.Popen(cmd, shell=True, stdout=tmpfile,
                            stderr=subprocess.STDOUT)
    proc.wait()
    
    os.close(tmpfile)
    
    if proc.returncode:
        log.error("Error running command %s" % cmd)
        log.error("Output: %s" % open(tmpfilename).read())
        raise SystemExit
    else:
        os.unlink(tmpfilename)

def exec_with_progress(cmd, metadata):
    """
        Execute a command cmd with a progress bar showing status. The progress
        bar will be rendered using the current time information output from
        e.g. ffmpeg vs. the total duration of the file in metadata.
    """
    log.debug("Running %s" % cmd)
    
    duration = metadata.duration
    start = time.time()
    tmpfile, tmpfilename = tempfile.mkstemp()
    
    log.debug("Tempfile is %s" % tmpfilename)
    
    proc = subprocess.Popen(cmd, shell=True, stdout=tmpfile, stderr=subprocess.STDOUT)
    
    output = ""
    while proc.poll() == None:
        time.sleep(1)
        
        tmp = None
        try:
            tmp = open(tmpfilename)
            tmp.seek(-100, 2)
            line = tmp.read().split("\n")[-1]
        except IOError:
            line = "time=0.0"
        finally:
            if tmp:
                tmp.close()
            
        result = time_re.search(line)
        if result:
            try:
                percent = max(0.0, min(1.0, float(result.groups()[0]) / duration))
            except ZeroDivisionError:
                percent = 0.0
                
            if percent == 0.0:
                rem = 0
            else:
                cur = time.time()
                rem = (cur - start) / (percent * 100) * (100 - (percent * 100))
            
            rem_str = " %s%% (%s)" % (str(int(percent * 1000) / 10.0), format_time(rem))
            draw_progress(percent, rem_str)
    
    clear_progress()
    
    os.close(tmpfile)
    
    if proc.returncode:
        log.error("Error running command %s" % cmd)
        log.error("Output: %s" % open(tmpfilename).read())
        raise SystemExit
    else:
        os.unlink(tmpfilename)

