#!/usr/bin/env python

"""
    STP Video Encoder & Uploader
    ============================
    A script to easily encode and upload videos. Output formats and upload 
    locations can be given based on regular expressions in the configuration
    file.
    
    Supported video formats include:
    
     * H.264/AAC in MP4 (for display online with e.g. JW FLV)
     * MPEG2 audio/video (for writing DVDs)
     * WMV2/WMA2 in ASF
    
    Supported upload protocols include:
    
     * sftp (FTP through SSH)
     * ftps (FTP encapsulated in TLS/SSL)
     * ftp (plain insecure FTP - don't use this)
    
    License
    -------
    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org> And Josh Chesarek <josh@simplethoughtproductions.com>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

import log
import os
import re
import shutil
import socket
import sre_constants
import subprocess
import sys
import tempfile
import threading
import time

from optparse import OptionParser

import libstpenc

if hasattr(sys, "frozen"):
    # This is needed for py2exe to work properly!
    PATH = os.path.dirname(unicode(sys.executable, sys.getfilesystemencoding()))
else:
    PATH = os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))
    PATH = os.path.abspath(PATH)

# Platform-specific functionality goes here!
if "win" in sys.platform and sys.platform != "darwin":
    # Microsoft Windows
    import atexit
    import msvcrt
    
    # Modify path to find ffmpeg.exe because spaces can't be passed to
    # os.system or subprocess.popen... *sigh*
    if PATH not in os.environ["PATH"]:
        os.environ["PATH"] = PATH + ";" + os.environ["PATH"]
    
    libstpenc.settings.BIN_FFMPEG = os.path.join(PATH, "ffmpeg.exe")
    libstpenc.settings.BIN_X264 = os.path.join(PATH, "x264.exe")
    
    # On Microsoft Windows the terminal (cmd.exe) will close upon completion
    # but we want it to stay open and display success or error messages, so
    # we wait for a key to be pressed or the window to be closed manually.
    def wait_for_key():
        print "Press any key to exit"
        while True:
            if msvcrt.kbhit():
                break
            time.sleep(0.25)
    
    atexit.register(wait_for_key)

# Make sure a configuration file exists
if not os.path.exists(os.path.join(PATH, "config.py")):
    if not os.path.exists(os.path.join(PATH, "config.default")):
        log.error("No default configuration file found!")
        raise SystemExit(1)
    
    log.info("Creating configuration file from defaults...")
    shutil.copy(os.path.join(PATH, "config.default"),
                os.path.join(PATH, "config.py"))

import config

if __name__ == "__main__":
    VERSION = libstpenc.settings.VERSION
    FORMATS = libstpenc.settings.FORMATS
    UPLOAD_PROTOCOLS = libstpenc.settings.UPLOAD_PROTOCOLS

    parser = OptionParser(usage="%prog [-f] infile", version="%prog " + VERSION)
    
    parser.add_option("-f", "--format", dest="format",
                      default=config.DEFAULT_FORMAT,
                      help="Manually specify a format to encode to [%s]" %\
                      config.DEFAULT_FORMAT)
    parser.add_option("-l", "--list-formats", dest="list_formats",
                      action="store_true", default=False,
                      help="Show list of available formats [false]")
    parser.add_option("-p", "--list-protocols", dest="list_protocols",
                      action="store_true", default=False,
                      help="Show list of available upload protocol formats [false]")
    parser.add_option("-s", "--scale", dest="scale",
                      default="x".join([str(item) for item in config.DEFAULT_SCALE]),
                      help="Width and height to scale video for manually set format, where -1 is auto [%s]" % \
                      "x".join([str(item) for item in config.DEFAULT_SCALE]))
    parser.add_option("-d", "--debug", dest="debug", default=config.DEBUG,
                      action="store_true",
                      help="Enable debug output [%s]" % str(config.DEBUG))
    #parser.add_option("-x", "--win32", dest="win32", default=False,
    #                  action="store_true",
    #                  help="Use win32 ffmpeg.exe regardless of current OS [false]")
    parser.add_option("-w", "--watch", dest="watch", default=config.DEFAULT_WATCH,
                      action="store_true",
                      help="Watch for creation of file and growth, then encode [%s]" % \
                      str(config.DEFAULT_WATCH))
    parser.add_option("-b", "--bitrate", dest="vbitrate", default=config.DEFAULT_VBITRATE,
                      help="Video bitrate for manually set format [%d kbps]" % config.DEFAULT_VBITRATE)
    parser.add_option("-a", "--audio-bitrate", dest="abitrate", default=config.DEFAULT_ABITRATE, 
                      help="Audio bitrate for manually set format [%d kbps]" % config.DEFAULT_ABITRATE)
    
    options, args = parser.parse_args()
    
    if options.debug:
        log.set_level("debug")
    else:
        log.set_format("%(levelname)s %(message)s")
    
    if options.format != "auto" and options.format not in FORMATS:
        log.error("Invalid format '%s' passed. Use --list-formats to see " \
                  "available formats!" % options.format)
        raise SystemExit(1)
    
    for regex in config.ACTIONS:
        try:
            re.compile(regex)
        except sre_constants.error:
            log.error("Invalid regular expression '%s' in configuration file!" % \
                      regex)
            raise SystemExit(1)
        
        for action in config.ACTIONS[regex]:
            try:
                profile = action[0]
                
                try:
                    format, scale, vbitrate, abitrate = profile
                except TypeError:
                    log.error("Invalid profile '%s' in configuration file! " \
                              "Profile must be a list: [\"format\", (width, " \
                              "height), vbitrate, abitrate]" % str(profile))
                
                uploads = action[1:]
            except TypeError:
                log.error("Invalid action '%s' in configuration file! " \
                          "Actions must be a list!" % str(action))
                raise SystemExit(1)
            
            if format not in FORMATS:
                log.error("Invalid format '%s' in configuration file! Use " \
                          "--list-formats to see available formats." % \
                          str(format))
                raise SystemExit(1)
            
            for upload_info in uploads:
                try:
                    service, path = upload_info
                except ValueError, TypeError:
                    log.error("Invalid upload '%s' in configuration file! " \
                              "Upload info should be a two-tuple " \
                              "('proto://user:pass@host:basepath', 'path')!" % \
                              str(upload_info))
                    raise SystemExit(1)
                
                result = libstpenc.uploaders.upload_re.search(service)
                if not result:
                    log.error("Invalid upload service '%s' in configuration " \
                              "file! Should be of the format " \
                              "proto://user:pass@host:basepath." % str(service))
                    raise SystemExit(1)
                
                if result.group("protocol") not in UPLOAD_PROTOCOLS:
                    log.error("Invalid upload protocol '%s' in configuration " \
                              "file! Use --list-protocols to see avaialbe " \
                              "upload protocols!" % result.group("protocol"))
                    raise SystemExit(1)
    
    if options.list_formats:
        print "Available formats:"
        formats = FORMATS.keys()
        formats.sort()
        for format in formats:
            print format
        raise SystemExit()
    
    if options.list_protocols:
        print "Available upload protocols:"
        for proto in UPLOAD_PROTOCOLS:
            print proto
        raise SystemExit()
    
    if len(args) < 1:
        parser.print_help()
        raise SystemExit(1)
    
    if not len(FORMATS):
        print "You have no formats defined! D'oh!"
        raise SystemExit(1)
    
    if options.watch:
        for filename in args:
            log.info("Waiting for file %s" % filename)
            
            lastsize = -1
            size = 0
            
            while not os.path.exists(filename):
                time.sleep(5)
                
            log.info("File created, watching for growth!")
            
            growing = True
            countdown = 0
            while True:
                lastsize = size
                try:
                    size = os.path.getsize(filename)
                except OSError:
                    log.error("Couldn't get file size!")
                    raise SystemExit(1)
                
                if growing and lastsize == size:
                    growing = False
                    countdown = GROWTH_CHECK_INTERVAL
                elif not growing and lastsize < size:
                    growing = True
                
                time_msg = ""
                if not growing:
                    countdown -= 1
                    
                    if not countdown:
                        break
                    
                    time_msg = " (grace period %s)" % format_time(countdown)
                
                msg = "Last file size was %d KiB%s" % (size / 1024, time_msg)
                
                sys.stdout.write("\010" * 79)
                sys.stdout.write(msg + " " * (79 - len(msg)))
                sys.stdout.flush()
                
                time.sleep(1)
            
            sys.stdout.write("\010" * 79)  
            log.info("File is ready!")
    else:
        for filename in args:
            if not os.path.exists(filename):
                log.error("File '%s' doesn't seem to exist..." % filename)
                raise SystemExit(1)
    
    if hasattr(options, "win32") and options.win32:
        BIN_FFMPEG = os.path.join(os.getcwd(), "ffmpeg.exe")
        log.debug("Using %s" % libstpenc.settings.BIN_FFMPEG)
    
    if "win" in sys.platform and sys.platform != "darwin":
        checkpath = os.path.join(PATH, libstpenc.settings.BIN_FFMPEG)
        if not os.path.exists(checkpath):
            print "%s is needed but was not found..." % checkpath
            raise SystemExit(1)
    
    for pos, filename in enumerate(args):
        log.info("Encoding file %d of %d: %s" % (pos + 1, len(args),
                                                    filename))
        infile = os.path.abspath(filename)
        prefix = ".".join(infile.split(".")[:-1])
        
        if options.format == "auto":
            for regex in config.ACTIONS.keys():
                if re.compile(regex).search(os.path.basename(filename)):
                    for action in config.ACTIONS[regex]:
                        # Get profile options
                        format, scale, vbitrate, abitrate = action[0]
                        
                        # Encode!
                        enc = FORMATS[format](infile, prefix, scale, vbitrate,
                                              abitrate)
                        enc.start()
                        
                        while enc.isAlive():
                            time.sleep(1.0)
                        
                        if len(action) > 1:
                            for upload_info in action[1:]:
                                # Is the outfile a single file or a list?
                                if type(enc.outfile) in [list, tuple]:
                                    for outfile in enc.outfile:
                                        upload(upload_info, outfile)
                                else:
                                    upload(upload_info, enc.outfile)
                    break
        else:
            enc = FORMATS[options.format](infile, prefix, options.scale,
                                          options.vbitrate, 
                                          options.abitrate)
            enc.start()
            
            while enc.isAlive():
                time.sleep(1.0)
    
    log.info("Done!")

